#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 16 10:29:52 2023

@author: gilles
"""




def bissextile(n):
    """
    Fonction bissextile

    Paramètres
    ----------
    n : entier
        numéro de l'année

    Renvoi
    -------
    booléen
        indique si l'année est bissextile ou non
    """
    return n%4 == 0 and (n % 100 !=0 or n%400==0)


def jour_suivant(m,an,jseed):
    """
    Fonction jour_suivant

    Paramètres
    ----------
    mois : entier
        numéro du mois de l'année précédente
    an : entier
        numéro de l'année précédente
    jseed : entier
        numéro du jour de la semaine précédent le premier du mois m

    Returns
    ------
    entier
        numéro du jour de la semaine du dernier jour du mois m

    """
    moisn={1:31,2:28,3:31,4:30,5:31,6:30,7:31,8:31,9:30,10:31,11:30,12:31} # nombre de jours par mois
    if m==2 and bissextile(an):
        nj=29
    else:
        nj=moisn[m]
    return (jseed+nj)%7



def sdl5(m,jseed):
    """
    Fonction sdl5

    Paramètres
    ----------
    mois : entier
        numéro du mois de l'année
    jseed : entier
        numéro du jour de la semaine précédent le premier du mois mois

    Renvoi
    -------
    booléen
        Vrai si le mois m comporte 5 samedis, 5 dimanches et 5 lundis
    """
    moisn={1:31,2:28,3:31,4:30,5:31,6:30,7:31,8:31,9:30,10:31,11:30,12:31} # nombre de jours par mois
    return jseed==4 and moisn[m]==31

def genere(jseed,m,annee,duree):
    """
    Fonction genere

    Paramètres
    ----------
    mois : entier
        numéro du mois de l'année précédente
    annee : entier
        numéro de l'année précédente
    jseed : entier
        numéro du jour de la semaine du premier du mois m
    duree : entier
        nombre d'années à étudier depuis l'année an

    Renvoi
    -------
    res : dictionnaire
        les clés sont les années et les valeurs sont les tableaux des mois qui comportent 5 samedis, 5 dimanches et 5 lundis

    """
    res={}
    jour=jseed-1
    nommois={1:"janvier", 2:"février", 3:"mars", 4:"avril", 5:"mai", 6 :"juin", 7 :"juillet", 8:"août", 9 :"septembre", 10 : "octobre", 11 :"novembre", 12 :"décembre"} # noms des mois de l'année
    for an in range(annee,annee+duree):
        while m < 13:
            if sdl5(m,jour):
                if an in res.keys():
                    res[an].append(nommois[m])
                else:
                    res[an]=[nommois[m]]
            jour=jour_suivant(m, an, jour)
            m += 1
        m=1
    return res

# Le 1 er janvier 2023 est un dimanche, donc, pour avoir les mois qui comptent 5 samedis, 5 dimanches et 5 lundis sur 20 ans  on saisit :

print(genere(6, 1, 2023, 20))